package be.iminds.ilabt.jfed.testing.base;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Properties;

/**
 * LegacyApiTestConfig
 */
public class LegacyApiTestConfig implements ApiTestConfig {
    @Nonnull private final Properties properties;
    @Nonnull private final LegacyApiTestMetaData legacyApiTestMetaData;

    public LegacyApiTestConfig(@Nonnull String propertiesString, @Nonnull LegacyApiTestMetaData legacyApiTestMetaData) {
        this.legacyApiTestMetaData = legacyApiTestMetaData;
        this.properties = new Properties();
        StringReader stringReader = new StringReader(propertiesString);
        try {
            properties.load(stringReader);
            stringReader.close();
        } catch (IOException e) {
            throw new RuntimeException("Unexpected IOException", e);
        }
    }

    public LegacyApiTestConfig(@Nonnull Properties properties, @Nonnull LegacyApiTestMetaData legacyApiTestMetaData) {
        this.legacyApiTestMetaData = legacyApiTestMetaData;
        this.properties = properties;
    }

    @Override
    public boolean isValid() {
        for (String reqKey : legacyApiTestMetaData.getReqConfKeys()) {
            if (!properties.containsKey(reqKey))
                return false;
        }
        return true;
    }

    @Nonnull
    @Override
    public String stringValue() {
        StringWriter stringWriter = new StringWriter();
        try {
            properties.store(stringWriter, "");
            return stringWriter.toString();
        } catch (IOException e) {
            throw new RuntimeException("Failed to convert properties to string", e);
        }
    }

    @Nullable public String getProperty(@Nonnull String key) {
        return properties.getProperty(key);
    }

    @Nonnull public String getProperty(@Nonnull String key, @Nonnull String defaultValue) {
        return properties.getProperty(key, defaultValue);
    }

    @Nullable public Object get(@Nonnull String key) {
        return properties.get(key);
    }

    //dangerous, you almost always need containsKey!
//    public boolean contains(String key) {
//        return properties.contains(key);
//    }
    public boolean containsKey(String key) {
        return properties.containsKey(key);
    }

    @Nonnull
    public Properties getProperties() {
        return properties;
    }

    public void setProperty(String key, String value) {
        properties.setProperty(key, value);
    }
}
